@@ -2,7 +2,7 @@
 #
 #  Net::Server::PreFork - Net::Server personality
 #  
-#  $Id: PreFork.pm,v 1.55 2001/07/04 06:07:54 rhandom Exp $
+#  $Id: PreFork.pm,v 1.5 2001/08/23 17:02:57 rhandom Exp $
 #  
 #  Copyright (C) 2001, Paul T Seamons
 #                      paul@seamons.com
@@ -21,9 +21,11 @@ package Net::Server::PreFork;
 
 use strict;
 use vars qw($VERSION @ISA $LOCK_EX $LOCK_UN);
-use POSIX ();
+use POSIX qw(WNOHANG);
 use Fcntl ();
 use Net::Server;
+use Net::Server::SIG qw(register_sig check_sigs);
+use IO::Select ();
 
 $VERSION = $Net::Server::VERSION; # done until separated
 
@@ -39,7 +41,9 @@ sub options {
 
   $self->SUPER::options($ref);
 
-  foreach ( qw(min_servers max_servers spare_servers max_requests max_dequeue
+  foreach ( qw(min_servers max_servers
+               min_spare_servers max_spare_servers
+               spare_servers max_requests max_dequeue
                check_for_dead check_for_waiting check_for_dequeue
                lock_file serialize) ){
     $prop->{$_} = undef unless exists $prop->{$_};
@@ -53,30 +57,61 @@ sub post_configure {
   my $self = shift;
   my $prop = $self->{server};
 
+  ### legacy check
+  if( defined($prop->{spare_servers}) ){
+    die "The Net::Server::PreFork argument \"spare_servers\" has been
+deprecated as of version '0.75' in order to implement greater child
+control.  The new arguments to take \"spare_servers\" place are
+\"min_spare_servers\" and \"max_spare_servers\".  The defaults are 5
+and 15 respectively.  Please remove \"spare_servers\" from your
+argument list.  See the Perldoc Net::Server::PreFork for more
+information.
+";
+  }
+
   ### let the parent do the rest
   ### must do this first so that ppid reflects backgrounded process
   $self->SUPER::post_configure;
 
   ### some default values to check for
-  my $d = {min_servers   => 5,      # min num of servers to always have running
-           max_servers   => 10,     # max num of servers to run
-           spare_servers => 1,      # num of extra servers to have lying around
-           max_requests  => 1000,   # num of requests for each child to handle
-           check_for_dead    => 30, # how often to see if children are alive
-           check_for_waiting => 20, # how often to see if extra children exist
+  my $d = {min_servers       => 5,    # min num of servers to always have running
+           max_servers       => 50,   # max num of servers to run
+           min_spare_servers => 2,    # min num of servers just sitting there
+           max_spare_servers => 10,   # max num of servers just sitting there
+           max_requests      => 1000, # num of requests for each child to handle
+           check_for_waiting => 10,   # how often to see we have to many servers
+           check_for_dead    => 30,   # how often to see if children are alive
            };
   foreach (keys %$d){
     $prop->{$_} = $d->{$_}
     unless defined($prop->{$_}) && $prop->{$_} =~ /^\d+$/;
   }
 
+  if( $prop->{min_spare_servers} > $prop->{max_spare_servers} ){
+    $self->fatal("Error: \"min_spare_servers\" must be less than "
+                 ."\"max_spare_servers\"");
+  }
+
+  if( $prop->{min_spare_servers} > $prop->{min_servers} ){
+    $self->fatal("Error: \"min_spare_servers\" must be less than "
+                 ."\"min_servers\"");
+  }
+
+  if( $prop->{max_spare_servers} >= $prop->{max_servers} ){
+    $self->fatal("Error: \"max_spare_servers\" must be less than "
+                 ."\"max_servers\"");
+  }
+
+  ### we will need to force a select handle
+  $prop->{multi_port} = 1;
+
   ### I need to know who is the parent
   $prop->{ppid} = $$;
 
 }
 
 
-### now that we are bound prepare serialization
+### now that we are bound, prepare serialization
 sub post_bind {
   my $self = shift;
   my $prop = $self->{server};
@@ -84,54 +119,45 @@ sub post_bind {
   ### do the parents
   $self->SUPER::post_bind;
 
-  ### set up serialization
-  if( defined($prop->{multi_port}) 
-      || defined($prop->{serialize})
-      || $^O =~ /solaris/i ){
-
-    ### clean up method to use for serialization
-    if( !defined($prop->{serialize}) 
-        || $prop->{serialize} !~ /^(flock|semaphore|pipe)$/ ){
-      $prop->{serialize} = 'flock';
-    }
-
-    ### set up lock file
-    if( $prop->{serialize} eq 'flock' ){
-      $self->log(3,"Setting up serialization via flock");
-      if( defined($prop->{lock_file}) ){
-        $prop->{lock_file_unlink} = undef;
-      }else{
-        $prop->{lock_file} = POSIX::tmpnam();
-        $prop->{lock_file_unlink} = 1;
-      }
-
-    ### set up semaphore
-    }elsif( $prop->{serialize} eq 'semaphore' ){
-      $self->log(3,"Setting up serialization via semaphore");
-      require "IPC/SysV.pm";
-      require "IPC/Semaphore.pm";
-      my $s = IPC::Semaphore->new(IPC::SysV::IPC_PRIVATE(),
-                                  1,
-                                  IPC::SysV::S_IRWXU() | IPC::SysV::IPC_CREAT(),
-                                  ) || $self->fatal("Semaphore error [$!]");
-      $s->setall(1) || $self->fatal("Semaphore create error [$!]");
-      $prop->{sem} = $s;
-
-    ### set up pipe
-    }elsif( $prop->{serialize} eq 'pipe' ){
-      pipe( _WAITING, _READY );
-      _READY->autoflush(1);
-      _WAITING->autoflush(1);
-      $prop->{_READY}   = *_READY;
-      $prop->{_WAITING} = *_WAITING;
-      print _READY "First\n";
+  ### clean up method to use for serialization
+  if( ! defined($prop->{serialize}) 
+      || $prop->{serialize} !~ /^(flock|semaphore|pipe)$/ ){
+    $prop->{serialize} = 'flock';
+  }
 
+  ### set up lock file
+  if( $prop->{serialize} eq 'flock' ){
+    $self->log(3,"Setting up serialization via flock");
+    if( defined($prop->{lock_file}) ){
+      $prop->{lock_file_unlink} = undef;
     }else{
-      $self->fatal("Unknown serialization type \"$prop->{serialize}\"");
+      $prop->{lock_file} = POSIX::tmpnam();
+      $prop->{lock_file_unlink} = 1;
     }
 
+  ### set up semaphore
+  }elsif( $prop->{serialize} eq 'semaphore' ){
+    $self->log(3,"Setting up serialization via semaphore");
+    require "IPC/SysV.pm";
+    require "IPC/Semaphore.pm";
+    my $s = IPC::Semaphore->new(IPC::SysV::IPC_PRIVATE(),
+                                1,
+                                IPC::SysV::S_IRWXU() | IPC::SysV::IPC_CREAT(),
+                                ) || $self->fatal("Semaphore error [$!]");
+    $s->setall(1) || $self->fatal("Semaphore create error [$!]");
+    $prop->{sem} = $s;
+    
+  ### set up pipe
+  }elsif( $prop->{serialize} eq 'pipe' ){
+    pipe( _WAITING, _READY );
+    _READY->autoflush(1);
+    _WAITING->autoflush(1);
+    $prop->{_READY}   = *_READY;
+    $prop->{_WAITING} = *_WAITING;
+    print _READY "First\n";
+    
   }else{
-    $prop->{serialize} = '';
+    $self->fatal("Unknown serialization type \"$prop->{serialize}\"");
   }
 
 }
@@ -151,6 +177,12 @@ sub loop {
 
   $self->log(3,"Beginning prefork ($prop->{min_servers} processes)\n");
 
+  ### keep track of the status
+  $prop->{tally} = {time       => time(),
+                    waiting    => 0,
+                    processing => 0,
+                    dequeue    => 0};
+
   ### start up the children
   $self->run_n_children( $prop->{min_servers} );
 
@@ -159,18 +191,38 @@ sub loop {
   
 }
 
+### sub to kill of a specified number of children
+sub kill_n_children {
+  my $self = shift;
+  my $prop = $self->{server};
+  my $n    = shift;
+  return unless $n > 0;
+
+  return unless time() - $prop->{last_kill} > 10;
+  $prop->{last_kill} = time();
+
+  $self->log(3,"Killing \"$n\" children");
+
+  foreach (keys %{ $prop->{children} }){
+    next unless $prop->{children}->{$_} eq 'waiting';
+    if( ! kill('HUP',$_) ){
+      delete $prop->{children}->{$_};
+      next;
+    }
+    last if --$n <= 0;
+  }
+}
+
 ### subroutine to start up a specified number of children
 sub run_n_children {
   my $self  = shift;
   my $prop  = $self->{server};
   my $n     = shift;
-  my $total = scalar keys %{ $prop->{children} };
-  
-  ### don't start more than we're allowed
-  if( $n > $prop->{max_servers} - $total ){
-    $n = $prop->{max_servers} - $total;
-  }
+  return unless $n > 0;
 
+  $self->log(3,"Starting \"$n\" children");
+  $prop->{last_start} = time();
+  
   for( 1..$n ){
     my $pid = fork;
 
@@ -181,6 +233,7 @@ sub run_n_children {
     ### parent
     }elsif( $pid ){
       $prop->{children}->{$pid} = 'waiting';
+      $prop->{tally}->{waiting} ++;
 
     ### child
     }else{
@@ -197,24 +250,20 @@ sub run_child {
   my $prop = $self->{server};
 
   ### restore sigs (turn off warnings during)
-  $SIG{INT} = $SIG{TERM} = $SIG{QUIT} = 'DEFAULT';
+  $SIG{INT} = $SIG{TERM} = $SIG{QUIT}
+    = $SIG{CHLD} = $SIG{PIPE} = 'DEFAULT';
 
   $self->log(4,"Child Preforked ($$)\n");
-  delete $prop->{children};
+
+  delete $prop->{$_} foreach qw(children tally last_start last_process);
 
   $self->child_init_hook;
 
-  ### tell the parent that we are waiting
-  *_WRITE = $prop->{_WRITE};
-  _WRITE->autoflush(1);
-  print _WRITE "$$ waiting\n";
-
   ### let the parent shut me down
   $prop->{connected} = 0;
   $prop->{SigHUPed}  = 0;
   $SIG{HUP} = sub {
     unless( $prop->{connected} ){
-      print _WRITE "$$ exiting\n";
       exit;
     }
     $prop->{SigHUPed} = 1;
@@ -315,8 +364,8 @@ sub run_parent {
   my $self=shift;
   my $prop = $self->{server};
 
-  $self->log(4,"Parent ready for children\n");
-
+  $self->log(4,"Parent ready for children.\n");
+  
   ### prepare to read from children
   local *_READ = $prop->{_READ};
   _READ->autoflush(1);
@@ -329,63 +378,81 @@ sub run_parent {
   $prop->{last_checked_for_dead}
   = $prop->{last_checked_for_waiting}
   = $prop->{last_checked_for_dequeue}
+  = $prop->{last_process}
+  = $prop->{last_kill}
   = time();
 
-  ### find the lowest positive time interval
-  my $alarm = (sort {$a <=> $b}
-               grep {defined($_) && $_>0} 
-               ($prop->{check_for_dead},
-                $prop->{check_for_waiting},
-                $prop->{check_for_dequeue},
-                ))[0];
-  
-  ### catch an alarm or a restart signal
-  ### do as little as possible here, work is done below
-  $SIG{ALRM} = sub{ $self->{sig} ||= 'alrm'; print _WRITE "$$ alrm\n"; };
-  $SIG{HUP}  = sub{ $self->{sig}   =  'hup'; print _WRITE "$$ hup\n";  };
+  ### register some of the signals for safe handling
+  register_sig(PIPE => 'IGNORE',
+               INT  => sub { $self->server_close() },
+               TERM => sub { $self->server_close() },
+               QUIT => sub { $self->server_close() },
+               HUP  => sub { $self->sig_hup() },
+               CHLD => sub {
+                 while ( defined(my $chld = waitpid(-1, WNOHANG)) ){
+                   last unless $chld > 0;
+                   delete $prop->{children}->{$chld};
+                 }
+               },
+### uncomment this area to allow SIG USR1 to give some runtime debugging               
+#               USR1 => sub {
+#                 require "Data/Dumper.pm";
+#                 print Data::Dumper::Dumper($self);
+#               },
+               );
+
+  ### need a selection handle
+  my $select = IO::Select->new();
+  $select->add(\*_READ);
 
   ### loop on reading info from the children
-  alarm($alarm);
-  while( <_READ> ){
-    alarm(0);
+  while( 1 ){
+
+    ### Wait to read.
+    ## Normally it is not good to do selects with
+    ## getline or <$fh> but this is controlled output
+    ## where everything that comes through came from us.
+    my @fh = $select->can_read(10);
+    if( &check_sigs() ){
+      last if $prop->{_HUP};
+    }
+    if( ! @fh ){
+      $self->coordinate_children();
+      next;
+    }
+
+    ### get the filehandle
+    my $fh = shift(@fh);
+    next unless defined $fh;
     
-    next if not defined $_;
+    ### read a line
+    my $line = <$fh>;
+    next if not defined $line;
 
     ### optional test by user hook
-    last if $self->parent_read_hook($_);
+    last if $self->parent_read_hook($line);
 
-    ### child should say "$pid waiting\n"
-    next unless /^(\d+)\ +(.+)$/;
+    ### child should say "$pid status\n"
+    next unless $line =~ /^(\d+)\ +(waiting|processing|dequeue|exiting)$/;
     my ($pid,$status) = ($1,$2);
 
-    ### allow for parent to get a word in edgewise
-    if( $pid == $$ || defined $self->{sig} ){
-      my $sig = delete($self->{sig}) || '';
-      if( $status eq 'alrm' ){
-        $self->coordinate_children();
-        alarm($alarm);
-        next;
-      }
-      if( $status eq 'hup' || $sig eq 'hup' ){
-        $self->sig_hup();
-        _READ->close();
-        last;
-      }
-    }
-
     ### record the status
-    if( $status eq 'exiting' ){
-      delete($prop->{children}->{$pid});
-    }else{
-      $prop->{children}->{$pid} = $status;
-    }
-    
-    ### check up on the children
-    if( $status eq 'processing' || $status eq 'exiting' ){
-      $self->coordinate_children();
+    $prop->{children}->{$pid} = $status;
+    if( $status eq 'processing' ){
+      $prop->{tally}->{processing} ++;
+      $prop->{tally}->{waiting} --;
+      $prop->{last_process} = time();
+    }elsif( $status eq 'waiting' ){
+      $prop->{tally}->{processing} --;
+      $prop->{tally}->{waiting} ++;
+    }elsif( $status eq 'exiting' ){
+      $prop->{tally}->{processing} --;
+      delete $prop->{children}->{$pid};
     }
 
-    alarm($alarm);
+    ### check up on the children
+    $self->coordinate_children();
+
   }
 
   ### allow fall back to main run method
@@ -401,6 +468,62 @@ sub coordinate_children {
   my $prop = $self->{server};
   my $time = time();
 
+  ### re-tally the possible types (only once a minute)
+  ### this might not be even necessary but is a nice sanity check
+  if( $time - $prop->{tally}->{time} > 6 ){
+    my $w = $prop->{tally}->{waiting};
+    my $p = $prop->{tally}->{processing};
+    $prop->{tally} = {time       => $time,
+                      waiting    => 0,
+                      processing => 0,
+                      dequeue    => 0};
+    foreach (values %{ $prop->{children} }){
+      $prop->{tally}->{$_} ++;
+    }
+    $w -= $prop->{tally}->{waiting};
+    $p -= $prop->{tally}->{processing};
+    $self->log(3,"Processing diff ($p), Waiting diff ($w)")
+      if $p || $w;
+  }
+
+  my $total = $prop->{tally}->{waiting} + $prop->{tally}->{processing};
+
+  ### need more min_servers
+  if( $total < $prop->{min_servers} ){
+    $self->run_n_children( $prop->{min_servers} - $total );
+    
+  ### need more min_spare_servers (up to max_servers)
+  }elsif( $prop->{tally}->{waiting} < $prop->{min_spare_servers} 
+          && $total < $prop->{max_servers} ){
+    my $n1 = $prop->{min_spare_servers} - $prop->{tally}->{waiting};
+    my $n2 = $prop->{max_servers} - $total;
+    $self->run_n_children( ($n2 > $n1) ? $n1 : $n2 );
+
+  }
+
+
+  ### check to see if we should kill off some children
+  if( $time - $prop->{last_checked_for_waiting} > $prop->{check_for_waiting} ){
+    $prop->{last_checked_for_waiting} = $time;
+
+    ### need fewer max_spare_servers (down to min_servers)
+    if( $prop->{tally}->{waiting} > $prop->{max_spare_servers}
+        && $total > $prop->{min_servers} ){
+
+      ### see if we haven't started any in the last ten seconds
+      if( $time - $prop->{last_start} > 10 ){
+        my $n1 = $prop->{tally}->{waiting} - $prop->{max_spare_servers};
+        my $n2 = $total - $prop->{min_servers};
+        $self->kill_n_children( ($n2 > $n1) ? $n1 : $n2 );
+      }      
+
+    ### how did this happen?
+    }elsif( $total > $prop->{max_servers} ){
+      $self->kill_n_children( $total - $prop->{max_servers} );
+      
+    }
+  }
+  
   ### periodically make sure children are alive
   if( $time - $prop->{last_checked_for_dead} > $prop->{check_for_dead} ){
     $prop->{last_checked_for_dead} = $time;
@@ -410,48 +533,24 @@ sub coordinate_children {
     }
   }
 
-  ### tally the possible types
-  my %num = (waiting=>0, processing=>0, dequeue=>0);
-  $num{$_} ++ foreach (values %{ $prop->{children} });
-  my $total = $num{waiting} + $num{processing};
-  
+  ### take us down to min if we haven't had a request in a while
+  if( $time - $prop->{last_process} > 30 && $prop->{tally}->{waiting} > $prop->{min_spare_servers} ){
+    my $n1 = $prop->{tally}->{waiting} - $prop->{min_spare_servers};
+    my $n2 = $total - $prop->{min_servers};
+    $self->kill_n_children( ($n2 > $n1) ? $n1 : $n2 );
+  }
+
   ### periodically check to see if we should clear the queue
   if( defined $prop->{check_for_dequeue} ){
     if( $time - $prop->{last_checked_for_dequeue} > $prop->{check_for_dequeue} ){
       $prop->{last_checked_for_dequeue} = $time;
       if( defined($prop->{max_dequeue})
-          && $num{dequeue} < $prop->{max_dequeue} ){
+          && $prop->{tally}->{dequeue} < $prop->{max_dequeue} ){
         $self->run_dequeue();
       }
     }
   }
 
-  ### need more min_servers
-  if( $total < $prop->{min_servers} ){
-    $self->run_n_children( $prop->{min_servers} - $total );
-    
-  ### need more spare_servers
-  }elsif( $num{waiting} < $prop->{spare_servers} 
-          && $total < $prop->{max_servers} ){
-    $self->run_n_children( $prop->{spare_servers} - $num{waiting} );
-    
-  ### need to remove some extra waiting servers
-  }elsif( $num{waiting} > $prop->{spare_servers} 
-          && $num{waiting} + $num{processing} > $prop->{min_servers} ){
-    if( $time - $prop->{last_checked_for_waiting} > $prop->{check_for_waiting} ){
-      $prop->{last_checked_for_waiting} = $time;
-      my $n = $num{waiting} + $num{processing} - $prop->{min_servers};
-      if( $n > $num{waiting} - $prop->{spare_servers} ){
-        $n = $num{waiting} - $prop->{spare_servers};
-      }
-      foreach (keys %{ $prop->{children} }){
-        next unless $prop->{children}->{$_} eq 'waiting';
-        last unless $n--;
-        kill(1,$_);
-      }
-    }
-    
-  }
 }
 
 
@@ -493,7 +592,7 @@ Net::Server::PreFork - Net::Server personality
      #...code...
   }
 
-  Net::Server::PreFork->run();
+  __PACKAGE__->run();
 
 =head1 DESCRIPTION
 
@@ -503,7 +602,7 @@ Net::Server module.
 
 This personality binds to one or more ports and then forks
 C<min_servers> child process.  The server will make sure
-that at any given time there are C<spare_servers> available
+that at any given time there are C<min_spare_servers> available
 to receive a client request, up to C<max_servers>.  Each of
 these children will process up to C<max_requests> client
 connections.  This type is good for a heavily hit site, and
@@ -522,8 +621,9 @@ configurable parameters.
 
   Key               Value                   Default
   min_servers       \d+                     5
-  spare_servers     \d+                     1
-  max_servers       \d+                     10
+  min_spare_servers \d+                     2
+  max_spare_servers \d+                     10
+  max_servers       \d+                     50
   max_requests      \d+                     1000
 
   serialize         (flock|semaphore|pipe)  undef
@@ -531,7 +631,7 @@ configurable parameters.
   lock_file         "filename"              POSIX::tmpnam
                                             
   check_for_dead    \d+                     30
-  check_for_waiting \d+                     20
+  check_for_waiting \d+                     10
 
   max_dequeue       \d+                     undef
   check_for_dequeue \d+                     undef
@@ -542,13 +642,22 @@ configurable parameters.
 
 The minimum number of servers to keep running.
 
-=item spare_servers
+=item min_spare_servers
 
 The minimum number of servers to have waiting for requests.
+Minimum and maximum numbers should not be set to close to
+each other or the server will fork and kill children too
+often.
+
+=item max_spare_servers
+
+The maximum number of servers to have waiting for requests.
+See I<min_spare_servers>.
 
 =item max_servers
 
-The maximum number of child servers to start.
+The maximum number of child servers to start.  This does not
+apply to dequeue processes.
 
 =item max_requests
 
@@ -585,12 +694,8 @@ without letting the parent know.
 
 =item check_for_waiting
 
-Seconds to wait before checking to see if there are too
-many waiting child processes.  Extra processes are killed.
-A time period is used rather than min_spare_servers and
-max_spare_server parameters to avoid constant forking and 
-killing when client requests are coming in close to the 
-spare server thresholds.
+Seconds to wait before checking to see if we can kill
+off some waiting servers.
 
 =item max_dequeue
 
@@ -620,7 +725,8 @@ white space are ignored.
   ### server information
   min_servers   20
   max_servers   80
-  spare_servers 10
+  min_spare_servers 10
+  min_spare_servers 15
 
   max_requests  1000
 
