@@ -1,22 +1,20 @@
 # -*- perl -*-
 #
 #  Net::Server::Proto::SSL - Net::Server Protocol module
-#
-#  $Id: SSL.pm,v 1.13 2010/05/05 03:13:03 rhandom Exp $
-#
-#  Copyright (C) 2001-2007
-#
-#    Paul Seamons
-#    paul@seamons.com
-#    http://seamons.com/
-#
+#  
+#  $Id: SSL.pm,v 1.2 2001/08/17 16:46:27 rhandom Exp $
+#  
+#  Copyright (C) 2001, Paul T Seamons
+#                      paul@seamons.com
+#                      http://seamons.com/
+#  
 #  This package may be distributed under the terms of either the
-#  GNU General Public License
+#  GNU General Public License 
 #    or the
 #  Perl Artistic License
 #
 #  All rights reserved.
-#
+#  
 ################################################################
 
 package Net::Server::Proto::SSL;
@@ -24,8 +22,7 @@ package Net::Server::Proto::SSL;
 use strict;
 use vars qw($VERSION $AUTOLOAD @ISA);
 use Net::Server::Proto::TCP ();
-eval { require IO::Socket::SSL; };
-$@ && warn "Module IO::Socket::SSL is required for SSL.";
+use IO::Socket::SSL ();
 
 $VERSION = $Net::Server::VERSION; # done until separated
 @ISA = qw(IO::Socket::SSL);
@@ -39,14 +36,10 @@ sub object {
   my $prop = $server->{server};
   my $host;
 
-  ### allow for things like "domain.com:80" (IPv4)
+  ### allow for things like "domain.com:80"
   if( $port =~ m/^([\w\.\-\*\/]+):(\w+)$/ ){
     ($host,$port) = ($1,$2);
 
-  ### allow for things like "[::1]:80" (IPv6)
-  }elsif( $port =~ m/^\[([\:\w\.\-\*\/]+)\]:(\w+)$/ ){
-    ($host,$port) = ($1,$2);
-
   ### allow for things like "80"
   }elsif( $port =~ /^(\w+)$/ ){
     ($host,$port) = ($default_host,$1);
@@ -56,40 +49,33 @@ sub object {
     $server->fatal("Undeterminate port \"$port\" under ".__PACKAGE__);
   }
 
-  # read any additional protocol specific arguments
-  my @ssl_args = qw(
-      SSL_server
-      SSL_use_cert
-      SSL_verify_mode
-      SSL_key_file
-      SSL_cert_file
-      SSL_ca_path
-      SSL_ca_file
-      SSL_cipher_list
-      SSL_passwd_cb
-      SSL_max_getline_length
-  );
-  my %args;
-  $args{$_} = \$prop->{$_} for @ssl_args;
-  $server->configure(\%args);
+  ### read any additional protocol specific arguments
+  $server->configure({
+    SSL_server      => \$prop->{SSL_server},
+    SSL_use_cert    => \$prop->{SSL_use_cert},
+    SSL_verify_mode => \$prop->{SSL_verify_mode},
+    SSL_key_file    => \$prop->{SSL_key_file},
+    SSL_cert_file   => \$prop->{SSL_cert_file},
+    SSL_ca_path     => \$prop->{SSL_ca_path},
+    SSL_ca_file     => \$prop->{SSL_ca_file},
+    SSL_cipher_list => \$prop->{SSL_cipher_list},
+  });
 
-  my $sock = $class->new;
+  ### create the handle under this package
+  my $sock = $class->SUPER::new();
+
+  ### store some properties
   $sock->NS_host($host);
   $sock->NS_port($port);
   $sock->NS_proto('SSL');
 
-  for my $key (@ssl_args) {
-    my $val = defined($prop->{$key}) ? $prop->{$key} : $server->can($key) ? $server->$key($host, $port, 'SSL') : undef;
-    $sock->$key($val);
-  }
-
   return $sock;
 }
 
 sub log_connect {
   my $sock = shift;
   my $server = shift;
-  my $host   = $sock->NS_host;
+  my $host   = $sock->NS_host; 
   my $port   = $sock->NS_port;
   my $proto  = $sock->NS_proto;
  $server->log(2,"Binding to $proto port $port on host $host\n");
@@ -116,7 +102,7 @@ sub connect {
     next unless /^SSL_/;
     $args{$_} = $prop->{$_};
   }
-
+  
   ### connect to the sock
   $sock->SUPER::configure(\%args)
     or $server->fatal("Can't connect to SSL port $port on $host [$!]");
@@ -144,7 +130,6 @@ sub accept {
 
   ### pass items on
   if( defined($client) ){
-    bless $client, ref($sock);
     $client->NS_proto( $sock->NS_proto );
   }
 
@@ -168,8 +153,8 @@ sub hup_string {
 sub show {
   my $sock = shift;
   my $t = "Ref = \"" .ref($sock) . "\"\n";
-  foreach my $prop ( qw(NS_proto NS_port NS_host) ){
-    $t .= "  $prop = \"" .$sock->$prop()."\"\n";
+  foreach ( qw(NS_proto NS_port NS_host) ){
+    $t .= "  $_ = \"" .$sock->$_."\"\n";
   }
   return $t;
 }
@@ -207,20 +192,17 @@ sub AUTOLOAD {
 
 =head1 NAME
 
-Net::Server::Proto::SSL - Net::Server SSL protocol (deprecated - use Net::Server::Proto::SSLEAY instead).
+  Net::Server::Proto::SSL - adp0 - Net::Server SSL protocol.
 
 =head1 SYNOPSIS
 
-This module is mostly deprecated - you will want to look at Net::Server::Proto::SSLEAY instead.
-
 See L<Net::Server::Proto>.
-See L<Net::Server::Proto::SSLEAY>.
 
 =head1 DESCRIPTION
 
-This original SSL module was experimental.  It has been superceeded by
-Net::Server::Proto::SSLEAY If anybody has any successes or ideas for
-improvment under SSL, please email <paul@seamons.com>.
+Experimental.  If anybody has any successes or ideas for
+improvment under SSL, please email <perl.ssl@seamons.com>.
+This is extremely alpha.
 
 Protocol module for Net::Server.  This module implements a
 secure socket layer over tcp (also known as SSL).
@@ -232,38 +214,14 @@ Net::Server.  However, Net::Server should also be able to
 maintain any number of TCP, UDP, or UNIX connections in
 addition to the one SSL connection.
 
-Additionally, getline support is very limited and writing directly to
-STDOUT will not work.  This is entirely dependent upon the
-implementation of IO::Socket::SSL.  getline may work but the client is
-not copied to STDOUT under SSL.  It is suggested that clients sysread
-and syswrite to the client handle (located in
-$self->{server}->{client} or passed to the process_request subroutine
-as the first argument).
-
 =head1 PARAMETERS
 
 In addition to the normal Net::Server parameters, any of the
 SSL parameters from IO::Socket::SSL may also be specified.
 See L<IO::Socket::SSL> for information on setting this up.
 
-=head1 BUGS
-
-Christopher A Bongaarts pointed out that if the SSL negotiation is
-slow then the server won't be accepting for that period of time
-(because the locking of accept is around both the socket accept and
-the SSL negotiation).  This means that as it stands now the SSL
-implementation is susceptible to DOS attacks.  To fix this will
-require deviding up the accept call a little bit more finely which may
-not yet be possible with IO::Socket::SSL.  Any ideas or patches on
-this bug are welcome.
-
 =head1 LICENCE
 
 Distributed under the same terms as Net::Server
 
-=head1 THANKS
-
-Thanks to Vadim for pointing out the IO::Socket::SSL accept
-was returning objects blessed into the wrong class.
-
 =cut
