@@ -1,32 +1,30 @@
 # -*- perl -*-
 #
 #  Net::Server::Proto::TCP - Net::Server Protocol module
-#
-#  $Id: TCP.pm,v 1.15 2010/05/05 04:41:08 rhandom Exp $
-#
-#  Copyright (C) 2001-2007
-#
-#    Paul Seamons
-#    paul@seamons.com
-#    http://seamons.com/
-#
+#  
+#  $Id: TCP.pm,v 1.7 2001/08/17 15:52:27 rhandom Exp $
+#  
+#  Copyright (C) 2001, Paul T Seamons
+#                      paul@seamons.com
+#                      http://seamons.com/
+#  
 #  This package may be distributed under the terms of either the
-#  GNU General Public License
+#  GNU General Public License 
 #    or the
 #  Perl Artistic License
 #
 #  All rights reserved.
-#
+#  
 ################################################################
 
 package Net::Server::Proto::TCP;
 
 use strict;
 use vars qw($VERSION $AUTOLOAD @ISA);
-use IO::Socket::INET6 ();
+use IO::Socket::INET ();
 
 $VERSION = $Net::Server::VERSION; # done until separated
-@ISA = qw(IO::Socket::INET6);
+@ISA = qw(IO::Socket::INET);
 
 sub object {
   my $type  = shift;
@@ -35,14 +33,10 @@ sub object {
   my ($default_host,$port,$server) = @_;
   my $host;
 
-  ### allow for things like "domain.com:80" (IPv4)
+  ### allow for things like "domain.com:80"
   if( $port =~ m/^([\w\.\-\*\/]+):(\w+)$/ ){
     ($host,$port) = ($1,$2);
 
-  ### allow for things like "[::1]:80" (IPv6)
-  }elsif( $port =~ m/^\[([\:\w\.\-\*\/]+)\]:(\w+)$/ ){
-    ($host,$port) = ($1,$2);
-
   ### allow for things like "80"
   }elsif( $port =~ /^(\w+)$/ ){
     ($host,$port) = ($default_host,$1);
@@ -87,16 +81,11 @@ sub connect {
   $args{LocalAddr} = $host if $host !~ /\*/; # what local address (* is all)
   $args{Listen}    = $prop->{listen};        # how many connections for kernel to queue
   $args{Reuse}     = 1;  # allow us to rebind the port on a restart
-
+  
   ### connect to the sock
   $sock->SUPER::configure(\%args)
     or $server->fatal("Can't connect to TCP port $port on $host [$!]");
 
-  if ($port == 0 && ($port = $sock->sockport)) {
-    $sock->NS_port($port);
-    $server->log(2,"Bound to auto-assigned port $port");
-  }
-
   $server->fatal("Back sock [$!]!".caller())
     unless $sock;
 
@@ -126,30 +115,6 @@ sub accept {
   return $client;
 }
 
-
-###----------------------------------------------------------------###
-
-sub read_until { # only sips the data - but it allows for compatibility with SSLEAY
-    my ($client, $bytes, $end_qr) = @_;
-    die "One of bytes or end_qr should be defined for TCP read_until\n" if !defined($bytes) && !defined($end_qr);
-    my $content = '';
-    my $ok = 0;
-    while (1) {
-        $client->read($content, 1, length($content));
-        if (defined($bytes) && length($content) >= $bytes) {
-            $ok = 2;
-            last;
-        }
-        elsif (defined($end_qr) && $content =~ $end_qr) {
-            $ok = 1;
-            last;
-        }
-    }
-    return wantarray ? ($ok, $content) : $content;
-}
-
-###----------------------------------------------------------------###
-
 ### a string containing any information necessary for restarting the server
 ### via a -HUP signal
 ### a newline is not allowed
@@ -167,8 +132,8 @@ sub hup_string {
 sub show {
   my $sock = shift;
   my $t = "Ref = \"" .ref($sock) . "\"\n";
-  foreach my $prop ( qw(NS_proto NS_port NS_host) ){
-    $t .= "  $prop = \"" .$sock->$prop()."\"\n";
+  foreach ( qw(NS_proto NS_port NS_host) ){
+    $t .= "  $_ = \"" .$sock->$_."\"\n";
   }
   return $t;
 }
@@ -208,7 +173,7 @@ __END__
 
 =head1 NAME
 
-  Net::Server::Proto::TCP - Net::Server TCP protocol.
+  Net::Server::Proto::TCP - adp0 - Net::Server TCP protocol.
 
 =head1 SYNOPSIS
 
